
/**
 * Classe représentant un polynôme par ses coefficients.
 * Toutes les opérations ont comme résultat le polynôme lui même, modifié par ladite opération.
 * Le polynôme nul n'a pas de coefficient ni de degré mais la propriété nul = vrai.
 */
var Polynome = function (coeffs) {
	//si on ne donne rien comme argument, le polynôme est nul.
	if (typeof coeffs === "undefined") {
		this.nul = true;
	} else if (coeffs instanceof Array) {
		//on a donné un vecteur de coefficients.
		if (coeffs.length == 0) {
			this.nul = true;
			return; //stop, il n'y a pas de coefficents.
		}
		this.nul = false;
		this.deg = coeffs.length-1;
		this.c = coeffs;
		//on peut construire des vecteurs incomplets en javascript. Un polynôme doit avoir tous ses coefficients, nuls éventuellement.
		for(var i = 0; i<= this.deg; i++) {
			if (this.c[i] == undefined) {
				this.c[i] = 0;
			}
		}
	}
};

Polynome.prototype = {
		degre: function() {
			if (this.nul) return undefined;
			return this.deg;
		},
		//ajoute le polynôme p au polynôme courant.
		plus: function(p) {
			if (p.nul) {
				//on ajoute 0...
				return this;
			}
			if (this.nul) {
				//on ajoute un "vrai" polynôme à 0
				this.deg = p.deg;
				this.c = p.c;
				this.nul = false;
				return this;
			}
			var i;
			if (p.deg > this.deg) {
				//si on ajoute un polynôme "plus grand" on rempli d'abord de 0 pour éviter undefined+cste.
				for (i = this.deg+1; i<= p.deg; i++) {
					this.c[i] = 0;
				}
				this.deg = p.deg;
			}
			for(i = 0; i <= p.deg; i++) {
				this.c[i] += p.c[i];
			}
			return p;
		},

		oppose: function() {
			if (this.nul) {
				return this;
			}
			for(var i = 0; i <= this.deg; i++) {
				this.c[i] = -this.c[i];
			}
			return this;
		},

		moins: function() {
			if (p.nul) {
				//on ajoute 0...
				return this;
			}
			if (this.nul) {
				//on ajoute un "vrai" polynôme à 0
				this.deg = p.deg;
				this.constructor = p.c;
				this.nul = false;
				return this.oppose();
			}
			var i;
			if (p.deg > this.deg) {
				//si on ajoute un polynôme "plus grand" on rempli d'abord de 0 pour éviter undefined+cste.
				for (i = this.deg+1; i<= p.deg; i++) {
					this.c[i] = 0;
				}
			}
			for(i = 0; i <= p.deg; i++) {
				this.c[i] -= p.c[i];
			}
			return this;
		},
		fois: function(p) {
			if (p.nul || this.nul) {
				//0 * polynôme = 0
				this.nul = true;
				delete this.deg;
				delete this.c;
				return this;
			}

			var i, s, newCoeffs = [];//variable pour calculer le coefficient d'indice k
			for(var k = 0; k <= this.deg + p.deg; k++) {
				s = 0;
				for(i = Math.max(0,k - p.deg); i <= Math.min(k, this.deg); i++) {
					//i est entre 0 et k et de plus i est entre 0 et this.deg, k-i est entre 0 et p.deg.
					s += this.c[i]*p.c[k-i];
				}
				newCoeffs[k] = s;
			}
			this.deg += p.deg;
			this.c = newCoeffs;
			return this;
		},
		//évaluation du polynôme en x par l'algorithme de Horner.
		eval: function(x) {
			if (this.nul) {
				return 0;
			}
			var s = 0;
			for(var i = this.deg; i >= 0 ; i--) {
				s *= x;
				s += this.c[i];
			}
			return s;
		}
};

/**
 * Retourne le polynôme d'interpolation de Lagrange corespondant aux valeurs données.
 * @param val un vecteur sous forme [[a0,b0],...,[an,bn]]. Les ai doivent être tous distincts.
 * @returns p un polynôme p de degré n tel que p(ai) = bi.
 */
function lagrange(val) {
	var p, j, v,
		l = new Polynome(); //l est le polynôme que l'on est en train de construire. 0 au début
	for(var i = 0; i < val.length; i++) {
		//on construit les polynôme de base Li. Li possède tous les aj comme racine sauf ai.
		p = new Polynome([1]); // polynôme constant égal à 1.
		for(j = 0; j < val.length; j++) {
			if (i !== j) {
				p.fois(new Polynome([-val[j][0],1])); //multiplie par x-aj si j!=i.
			}
		}
		v = p.eval(val[i][0]);
		if (v == 0) {//cas impossible en théorie
			console.log('erreur');
		} else {
			l.plus(p.fois(new Polynome([val[i][1]/v])));
		}
	}
	return l;
}
/**
 * Retourne un polynôme de degré n qui prend n+1 valeurs communes avec la fonction f, équirépartie dans l'intervalle donnée.
 * @param f
 * @param intervalle
 * @param n
 */
function polynomeInterpolateur(f, intervalle, n) {
	if (intervalle.debut > intervalle.fin || n == 0) {
		return null;
	}
	var a = new Array(n+1), i,
		step = (intervalle.fin - intervalle.debut)/n;
	for(i = 0; i <=n; i++) {
		a[i] = [intervalle.debut + i*step, f.eval(intervalle.debut + i*step)];
	};
	return lagrange(a);
}

//calcule n!. Retient tous les résultats intermédiaires.
var factorielles = [1];
function facto(n) {
	if (factorielles[n] != undefined) {
		return factorielles[n];
	}
	for(var i = 1; i<= n; i++) {
		if(factorielles[i] == undefined) {
			factorielles[i] = i * factorielles[i-1];
		}
	}
	return factorielles[n];
}

var repere,
	lecture = false,
	nbPoints = 0,
	intervalle = {debut:-5 ,fin: 5},
	fonc = [{
		eval: function(x) {return Math.sin(x);},
		dl: function(n) {
			var a = new Array();
			for(var k=0; 2*k+1 <= n; k++) {
				a[2*k+1] = ((k%2 == 0)?1:-1)*1/facto(2*k+1);

			}
			return (new Polynome(a));
		}},
        {eval: function(x) {return x*Math.sin(x)/3;},
			dl: function(n) {
				var a = new Array();
				for(var k=0; k*2+2<= n; k++) {
					a[2*k+2] = ((k%2 == 0)?1:-1)*1/facto(2*k+1)/3;
				};
				return new Polynome(a);
			}},
        {eval: function(x) {return 1/(1+x*x);},
			dl: function(n) {
				var a = new Array();
				for(var k=0; k*2 <= n; k++) {
					a[2*k] = ((k%2 == 0)?1:-1)*1;
				};
				return new Polynome(a);
			}},
        {eval: function(x) {return Math.sin(x*x) + Math.cos(x);},
			dl: function(n) {
				var a = new Array();
				for(var k=0; k*2 <= n; k++) {
					a[2*k] = ((k%2 == 0)?1:-1)*1/facto(2*k);;
				};
				for(k = 0; 4*k+2 <= n; k++) {
					a[4*k+2] += ((k%2 == 0)?1:-1)*1/facto(2*k+1);
				}
				return new Polynome(a);
			}}
        ],
    f = fonc[0],
    ecran;

function dessine() {
	repere.dessine(true, function() {
		var ctx = ecran.getContext('2d');
		ctx.clearRect(0, 0, ecran.width, ecran.height);
		ctx.drawImage(repere.c, 0,0, ecran.width, ecran.height);
	}, intervalle);
}

function loadFonc(index) {
	f = fonc[index];
	repere.assigneFonction(0,f);
	repere.supprimeFonction(1);
	dessine();
}

$(document).ready( function () {
var c = document.getElementById("backup"),
	mode = 0;
	ecran = document.getElementById("canvas");
	repere = new Repere(c,{yMin: -5, yMax: 5, xMin: -5.5, xMax: 5.5, nbPoints: 300});
	repere.ajouteFonction(f);
	dessine();


//La suite nécessite repere.js
document.getElementById("lecture").onclick = function() {
	if (lecture) {
		clearInterval(lecture);
		lecture = false;
	} else {
		lecture = setInterval( function () {
			nbPoints = (nbPoints + 1) % 16;
			if (mode === 0) {
				repere.assigneFonction(1, polynomeInterpolateur(f,intervalle, 5 + nbPoints));
			} else {
				repere.assigneFonction(1, f.dl(5 + nbPoints));
			}
			dessine();
			var ctx = ecran.getContext('2d');
			ctx.fillText("n = " + (3+nbPoints), 40 , 40);
		}, 500);
	}
};
$('.selectFonction').click( function () {
	loadFonc(parseInt($(this).data('fonc')));
});
$('.mode').click(function () {
	if ($(this).hasClass('activeMode')) {
		return;
	}
	$(".mode").toggleClass('activeMode');
	mode = parseInt($(this).data('mode'));
	repere.supprimeFonction(1);
	dessine();
});
var nbPoint = document.createElement('input');
nbPoint.type = 'text';
nbPoint.id = 'nbPoint';
$('#body').append($('<div>').append(nbPoint));

$("#nbPoint").change( function() {
	var n = parseInt(this.value);
	if (mode === 0) {
		repere.assigneFonction(1, polynomeInterpolateur(f,intervalle, n));
	} else {
		repere.assigneFonction(1, f.dl(n));
	}
	dessine();
});
});
